@@ -1,9 +1,12 @@
 import adal
+from azure.identity import DefaultAzureCredential
 from azure.keyvault.secrets import SecretClient
 from azure.identity import ManagedIdentityCredential
 from datetime import datetime
 from email.mime.text import MIMEText
 from email.mime.multipart import MIMEMultipart
+import json
+import os
 import pyodbc
 from sqlalchemy import create_engine
 import struct
@@ -146,51 +149,6 @@ def createSQLconnection(CREDS,LOGGER):
         LOGGER.error(f"mstAst.createSQLconnection Error: "+str(ec))
         return None
         
-def getAssetsMimir(CREDS,LOGGER):
-    try:
-        LOGGER.debug("Starting getAssetsMimir")
-        errors=0
-        while errors<3:
-            try:
-                conn=createSQLconnection(CREDS, LOGGER)
-                cursor = conn.cursor()
-                sql="SELECT [ID],[NAME],[PROJECT_CODE],[FRM_ASSET_ID],[PRIORITY],[IS_ACTIVE],[COUNTRY_ID],[LONGITUDE],[LATITUDE],[ENERGY_TYPE]"
-                sql+=",[PLANT_TYPE],[POWER_AC],[POWER_DC],[MONITORING_SYSTEM_ID],[NETWORK_OPERATOR],[SERVICE_COMPANY_ID] FROM [curated].[ASSET_MAIN];"
-                cursor.execute(sql)
-                assetsMimir={}
-                row = cursor.fetchone()
-                while row:
-                    assetsMimir[row[0]]={}
-                    assetsMimir['name']=row[1]
-                    assetsMimir['projectCode']=row[2]
-                    assetsMimir['FRMID']=row[3]
-                    assetsMimir['priority']=row[4]
-                    assetsMimir['active']=row[5]
-                    assetsMimir['countryId']=row[6]
-                    assetsMimir['longitude']=row[7]
-                    assetsMimir['latitude']=row[8]
-                    assetsMimir['energyType']=row[9]
-                    assetsMimir['plantType']=row[10]
-                    assetsMimir['powerAC']=row[11]
-                    assetsMimir['powerDC']=row[12]
-                    assetsMimir['monSysId']=row[13]
-                    assetsMimir['gridOperator']=row[14]
-                    assetsMimir['serviceCompanyId']=row[15]
-                    #assetsMimir['plantType']=row[16]
-                    #assetsMimir['plantType']=row[17]
-                    #assetsMimir['plantType']=row[18]
-                    #assetsMimir['plantType']=row[19]
-                    row = cursor.fetchone()
-                return assetsMimir
-            except:
-                errors+=1
-                time.sleep(0.05)
-        LOGGER.error(f"mstAst.getAssetsMimir Failed 3 times, stopping execution")
-        return None
-    except Exception as e1:
-        LOGGER.error(f"mstAst.getAssetsMimir Failed, error: {e1:[:1000]}")
-        return None
-        
 def getAssetParameter(DB_CRED,assetId,parameter,logger,subParameter=None):
     try:
         logger.debug("Starting getAssetParameter")
@@ -305,19 +263,15 @@ def getSetting(CREDS,version,settingName,systemShortName,logger):
         
 def dfToDB2(CREDS,table,df,chunkSize,logger):
     try:
-        #logger.debug(f"Starting mstAst4.dfToDB to table {table}")
-        #logger.debug(f"Creating connection")
         try:
             timeout=str(CREDS['timeout'])
         except:
             timeout='60'
         logger.debug(f"Creating connection string")
         conn_str=urllib.parse.quote_plus(r'Driver='+CREDS['driver']+';Server=tcp:'+CREDS['db_server']+',1433;Database='+CREDS['database']+';Uid='+CREDS['db_user']+';Pwd='+CREDS['db_password']+';Encrypt=yes;TrustServerCertificate=yes;Connection Timeout='+timeout+';')
-        #logger.debug(f"Connection string: {conn_str}")
         conn='mssql+pyodbc:///?odbc_connect={}'.format(conn_str)
         engine=create_engine(conn, fast_executemany=True, echo=False)
         #engine=create_engine(conn, fast_executemany=False, echo=False)
-        #logger.debug(f"Executing df.to_sql")
         #result=df.to_sql(table, engine, if_exists='append', chunksize=chunkSize, index=False, method='multi')
         logger.debug(f"dfToDB2 executing")
         timestampStart = datetime.now()
@@ -335,21 +289,17 @@ def dfToDB2(CREDS,table,df,chunkSize,logger):
             logger.debug(f"mstAst.dfToDB2 storing to table {table} ended with result: {result}, size: {size}, returning 0")
         return 0
     except Exception as e1:
-        log_text=f"mstAst.dfToDB2, storing to table {table} failed, error: "+str(e1)[:5000]
-        logger.error(log_text)
-        #logger.error(e1)
-        #raise
-        #raise error.with_traceback(sys.exc_info()[2])
+        logger.error(f"mstAst.dfToDB2, storing to table {table} failed, error: "+str(e1)[:5000])
         return 1
-        
-def getVaultSecret(CREDS,vaultName,secretName,logger):
+'''
+def getVaultSecret0(CREDS,vaultName,secretName,logger):
     try:
         logger.debug("Starting getVaultSecret")
         #keyVaultName = "dv-kv-monsys"
         #secretName="dv-sql"
         KVUri = f"https://{vaultName}.vault.azure.net"
         #credential = DefaultAzureCredential()
-        credential = ManagedIdentityCredential(client_id="031ce06f-fd5a-41a4-a62c-8ad477e71e98")
+        credential = ManagedIdentityCredential(client_id="test")
         client = SecretClient(vault_url=KVUri, credential=credential)
         secret = client.get_secret(secretName)
         return secret.value
@@ -357,3 +307,74 @@ def getVaultSecret(CREDS,vaultName,secretName,logger):
         log_text=f"mstAst.getVaultSecret Failed, error: {e1}[:500]"
         logger.error(log_text)
         return None
+'''
+    
+def getVaultSecret(vault_name,secret_name,logger):
+    try:
+        logger.debug("Starting getVaultSecret")
+        credential = DefaultAzureCredential()
+        client = SecretClient(vault_url=f"https://{vault_name}.vault.azure.net", credential=credential)
+        return client.get_secret(secret_name).value
+    except Exception as e1:
+        log_text=f"mstAst.getVaultSecret Failed, error: {e1}[:500]"
+        logger.error(log_text)
+        return None
+
+def insertSLALog(CREDS,service,function,factor,ex_time,logger):
+    try:
+        errors=0
+        while errors<3:
+            try:
+                with pyodbc.connect('Driver='+CREDS['driver']+';Server=tcp:'+CREDS['db_server']+',1433;Database='+CREDS['database']+';Uid='+CREDS['db_user']+';Pwd={'+CREDS['db_password']+'};Encrypt=yes;TrustServerCertificate=no;Connection Timeout=30') as conn:
+                    with conn.cursor() as cursor:
+                        if ex_time<1:#if a quick function, we set it to 1 second anyway
+                            ex_time=1
+                        if isinstance(service, int):#in case of use a hard-coded id, like in ghppun
+                            sql=f"DECLARE @serviceid INT = {service};"
+                        else:
+                            sql=f"DECLARE @serviceid INT = (SELECT [serviceId] FROM [dbo].[mstServices] WHERE [serviceName]='{service}');"
+                        sql+=f"INSERT INTO [dbo].[afaUsageLog] ([TS_INSERTED],[azureFunctionName],[serviceId],[costFactor],[executionTime]) "
+                        sql+=f"VALUES (CURRENT_TIMESTAMP, '{function}', @serviceid, {factor}, {ex_time});"
+
+                        logger.debug(f"mstAst.insertSLALog / sql: {sql}")
+                        cursor.execute(sql)
+                        rows=int("{}".format(cursor.rowcount))
+                        logger.debug(f"mstAst.insertSLALog / Received {rows} rows from cursor")
+                        if rows==0:
+                            #logger.debug(f"mstAst.executeCommand / 0 rows from cursor, returning 1")
+                            return 1#so that we know that execution was successful
+                        return rows
+            except Exception as s1:
+                if errors==0:
+                    logger.debug(f"mstAst.insertSLALog / first failure: "+str(s1)+", command: "+sql)
+                errors+=1
+                time.sleep(0.05)
+        logger.error("mstAst.executeCommand Failed 3 times, stopping execution, last command: "+sql)
+        return -1
+    except Exception as e1:
+        logger.error(f"mstAst.executeCommand Failed, error: {e1.args[1]}, service: {service}")
+        return -1
+    
+def writeFileDBX(path, file_name, data, logger, useJson=False):
+    try:
+        bytes_written=-1
+        logger.debug(f"Writing file {file_name} to {path}")
+        full_path = rf'{path}/{file_name}'
+        if not os.path.isdir(path):
+            logger.debug(f"Creating folder {path}")
+            os.makedirs(path, exist_ok=True)
+        with open(full_path, mode='w', encoding="utf-8") as f:
+            if useJson:
+                json.dump(data, f)
+                bytes_written=len(str(data))
+            else:
+                bytes_written=f.write(data)
+            k="bytes"
+            if bytes_written>10000:
+                bytes_written/=1024
+                k="kB"
+            logger.debug(f"File written ({bytes_written} {k})")
+    except Exception as e:
+        logger.error(f"mstAst.writeFileDBX Failed, error: "+str(e))
+    finally:
+        return bytes_written
\ No newline at end of file
